#include "main.h"

#include <kaboutdata.h>
#include <QSettings>
#include <QChar>
#include <QMessageBox>
#include <QtGlobal>
#include <QFile>
#include <KPluginFactory>
#include <KOpenWithDialog>

const int KReslimitConfig::DEFAULT_MEMSIZE = 1024;

K_PLUGIN_FACTORY(KReslimitFactory, registerPlugin<KReslimitConfig>();)
K_EXPORT_PLUGIN(KReslimitFactory("kreslimit"))

KReslimitConfig::KReslimitConfig(QWidget *parent, const QVariantList& args)
    : KCModule(parent, args)
{
    setButtons(KCModule::Apply | KCModule::Default);
    KAboutData *about =
        new KAboutData(QStringLiteral("kreslimit"), i18n("KDE Memory Limit Tool"), QStringLiteral("1.0.0"));

    setAboutData(about);

    ui.setupUi(this);

    connect(ui.all_enable, &QCheckBox::clicked, this, &KReslimitConfig::configChanged);
    connect(ui.all_memsize, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &KReslimitConfig::configChanged);
    connect(ui.app_add, &QPushButton::clicked, this, &KReslimitConfig::appAdd);
    connect(ui.app_remove, &QPushButton::clicked, this, &KReslimitConfig::appRemove);
    connect(ui.select_change, &QPushButton::clicked, this, &KReslimitConfig::applyChange);
    connect(ui.app_choose, &QPushButton::clicked, this, &KReslimitConfig::appChooseDialog);
    // set tooltips
    ui.all_enable->setToolTip(i18n("Enable memory limit for all running applications"));
    ui.all_memsize->setToolTip(i18n("Set memory limit size for all running applications"));
    ui.app_input->setToolTip(i18n("Enter the full path to the binary file of the application that needs to limit the size of available memory. Example: /usr/bin/firefox"));
    ui.app_memsize->setToolTip(i18n("Set memory limit size for selected application"));

    ui.app_enable->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    model = new KReslimitTableModel();
    ui.app_enable->setModel(model);
    connect(ui.app_enable->selectionModel(), &QItemSelectionModel::selectionChanged, this, &KReslimitConfig::selectChange);
}

KReslimitConfig::~KReslimitConfig()
{
}

QString KReslimitConfig::getPath(const QString app) {
    QByteArray ba = qgetenv("PATH");
    QStringList paths = QString::fromLocal8Bit(ba).split(':');
    for (int i = 0; i < paths.size(); ++i) {
        QString p = paths.at(i) + "/" + app;
        if(QFile::exists(p)) {
            return p;
        }
    }
    return app;
}

void KReslimitConfig::inputClear() {
    ui.app_input->clear();
    ui.app_memsize->setValue(DEFAULT_MEMSIZE);
    return;
}

void KReslimitConfig::appChooseDialog() {
    KOpenWithDialog dialog(this);
    dialog.setWindowModality(Qt::WindowModal);
    if (dialog.exec())
    {
        KService::Ptr service = dialog.service();
        if (service)
        {
            QString commandName = service->exec();
            ui.app_input->setText(getPath(commandName.split(' ')[0]));
        }
    }
}

void KReslimitConfig::configChanged() {
    emit changed(true);
}

void KReslimitConfig::save() {
    QSettings settings(QStringLiteral("kreslimit"));
    settings.setValue("all_enable", ui.all_enable->isChecked());
    settings.setValue("all_memsize", ui.all_memsize->value());
    settings.setValue("applist", model->applist.join(QChar::fromLatin1(',')));
    settings.sync();
}

void KReslimitConfig::load() {
    QSettings settings(QStringLiteral("kreslimit"));
    ui.all_enable->setChecked(settings.value("all_enable", false).toBool());
    ui.all_memsize->setValue(settings.value("all_memsize", QVariant(DEFAULT_MEMSIZE)).toInt());
    QString apps = settings.value("applist", QString()).toString();
    if(apps.isEmpty() || apps.isNull()) {
        model->applist = QStringList();
    } else {
        model->applist = apps.split(',');
    }
    model->refresh();
}

void KReslimitConfig::defaults() {
    ui.all_enable->setChecked(false);
    ui.all_memsize->setValue(DEFAULT_MEMSIZE);
    ui.app_memsize->setValue(DEFAULT_MEMSIZE);
}

void KReslimitConfig::appAdd() {
    if(ui.app_input->text().isEmpty()) {
        return;
    }
    QStringList::const_iterator constIterator;
    QString bin_path = ui.app_input->text();
    for (constIterator = model->applist.constBegin(); constIterator != model->applist.constEnd(); ++constIterator) {
        QString rec = *constIterator;
        if(rec.startsWith(bin_path)) {
            QStringList res = rec.split(QChar::fromLatin1(';'));
            if(bin_path == res[0]) {
                QMessageBox msgBox;
                msgBox.setText(i18n("Application has already been added!"));
                msgBox.exec();
                return;
            }
        }
    }
    model->applist.append(bin_path + ";" + QString::number(ui.app_memsize->value()));
    model->refresh();
    configChanged();
    inputClear();
    return;
}

void KReslimitConfig::appRemove() {
    QModelIndexList selection = ui.app_enable->selectionModel()->selectedRows();
    for (int i = 0; i < selection.size(); ++i) {
        model->applist.replace(selection.at(i).row(), "");
    }
    model->applist.removeAll("");
    model->refresh();
    configChanged();
    inputClear();
    return;
}

void KReslimitConfig::applyChange() {
    QModelIndexList selection = ui.app_enable->selectionModel()->selectedRows();
    if(selection.count() != 1)  {
        return;
    }
    QStringList::const_iterator constIterator;
    for (constIterator = model->applist.constBegin(); constIterator != model->applist.constEnd(); ++constIterator) {
        QString rec = *constIterator;
        if(rec.startsWith(ui.app_input->text())) {
            QStringList res = rec.split(QChar::fromLatin1(';'));
            if(ui.app_input->text() == res[0] && selection.at(0).row() != constIterator - model->applist.begin()) {
                QMessageBox msgBox;
                msgBox.setText(i18n("Application has already been added!"));
                msgBox.exec();
                return;
            }
        }
    }
    model->applist[selection.at(0).row()] = ui.app_input->text() + ";" + QString::number(ui.app_memsize->value());
    model->refresh();
    configChanged();
    inputClear();
    return;
}

void KReslimitConfig::selectChange() {
    QModelIndexList selection = ui.app_enable->selectionModel()->selectedRows();
    if(selection.count() == 1) {
        QStringList res = model->applist[selection.at(0).row()].split(QChar::fromLatin1(';'));
        ui.app_input->setText(res[0]);
        ui.app_memsize->setValue(res[1].toInt());
    } else {
        inputClear();
    }
    return;
}

#include "main.moc"
